function [MI, meanAmp] = f_modulationIndex(rawData,sampRate,winLen,stepLen,phaseFreq,ampFreq)
%Measure phase-amplitude cross frequency coupling using Modulation Index (MI)
%  proposed by Tort et al., Proc Natl Acad Sci, 2008.
%
%  Inputs:
%       rawData: one-dimensional raw signal (1xn or nx1)
%       sampRate: sampling rate of raw signal
%       winLen: Length of moving window in second
%       stepLen: step of moving window in second
%       phaseFreq: cutoff frequencies for phase frequency
%       ampFreq: cutoff frequencies for amplitude frequency
%  Outputs:
%       MI: modulation index value
%       meanAmp: Average power of high frequencies in each phase-bin
%
%  Mojtaba Bandarabadi, Inselspital, Bern, March 2015
%  mojtaba.bandarabadi@insel.ch
%

%%
if size(rawData,1)>size(rawData,2)
    rawData = rawData';
end

%% Extract phase for phase frequency band using Hilbert transform
order = round(3*(sampRate/phaseFreq(1)));
fir1Coef = fir1(order,[phaseFreq(1),phaseFreq(2)]./(sampRate/2));
phaseSignal = filtfilt(fir1Coef, 1, rawData);
Phase = angle(hilbert(phaseSignal));

%% Extract envelope for amplitude frequency band using Hilbert transform
order = round(3*(sampRate/ampFreq(1)));
fir1Coef = fir1(order,[ampFreq(1),ampFreq(2)]./(sampRate/2));
ampSignal = filtfilt(fir1Coef, 1, rawData);
Amp = abs(hilbert(ampSignal));

%% Dividing phase [0 2*pi] into several equal bins
nBin = 18; % 18 bins, each bin covers 20 degrees
binStart = zeros(1, nBin);
binSize = 2*pi/nBin;
for i = 1:nBin
    binStart(i) = (i-1)*binSize-pi; % start of each bin
end

%% Compute Modulation Index for each segment
nSeg = floor((length(Phase)-(winLen-stepLen)*sampRate)/(stepLen*sampRate)); % number of segments

MI = zeros(1, nSeg);
meanAmp = zeros(nSeg, nBin);
for j = 1:nSeg
    idx = (j-1)*stepLen*sampRate+1:((j-1)*stepLen+winLen)*sampRate; % index of segment
    AmpT = Amp(1,idx);
    PhaseT = Phase(1,idx);
    meanAmpT = zeros(1,nBin); % Mean power in each phase-bin
    for k = 1:nBin
        meanAmpT(k) = nanmean(AmpT(PhaseT>=binStart(k) & PhaseT<(binStart(k)+binSize)));
    end
    meanAmpT = meanAmpT./sum(meanAmpT); % Normalize distribution
    KLdist = meanAmpT.*log(nBin.*meanAmpT); % Kullback?Leibler (KL) distance
    KLdist(isnan(KLdist)) = 0;
    MI(1,j) = sum(KLdist)./log(nBin);
    meanAmp(j,:) = meanAmpT;
end
end